#!/bin/bash
CERTFILE="/usr/lib/eidconfig-belgium/Belgium_Root_CA_Self-signed.cer"
EXPECTFILE="/usr/lib/eidconfig-belgium/changepin.exp"
PKCS11MOD="/usr/lib/pkcs11/libbeidpkcs11.so"
PKCS11TOOL="beid-pkcs11-tool"
PKCS11NAME="Belgium Identity Card PKCS#11"

FFBASE="$HOME""/.mozilla/firefox"
OOBASE="$HOME""/.ooo-2.0/user"
EVOBASE="$HOME""/.evolution"

OO="false"
GOON="true"
PIN=""
CALL=""
ARG=""
PROFILE=""
OCSP=""
OCSP_LINE=""
CRL=""
CRL_LINE=""

########################################################

function enable_secmoddb_dir ()
{
if [ -f ${1}/secmod.db ] ; then
	add_beleid_to_secmod $1
else
	create_new_databases $1
	add_beleid_to_secmod $1
fi

if [ $OO = "true" ] ; then
	if [ -f ~/.bash_profile ] ; then
		echo "export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user" >> ~/.bash_profile
		echo "export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user" >> ~/.profile
	else
		echo "export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user" >> ~/.profile
	fi
fi
}

function create_new_databases ()
{
modutil -force -create -dbdir $1
}

function add_beleid_to_secmod ()
{
if modutil -force -add "$PKCS11NAME" -libfile $PKCS11MOD -dbdir $1; then
	:
else
	modutil -force -delete "$PKCS11NAME" -dbdir $1
	if modutil -force -add "$PKCS11NAME" -libfile $PKCS11MOD -dbdir $1; then
		:
	else
		exit 255
	fi
fi

if certutil -A -n "Belgium Root Self Signed" -t "CT,CT,CT" -d $1 -a -i $CERTFILE ; then
	:
else	
	if certutil -A -n "Belgium Root Self Signed" -t "CT,CT,CT" -d $1 -a -i $CERTFILE ; then
		:
	else	
		disable_secmoddb_dir $1
		exit 255
	fi
fi
}



#######################################################

function remove_beleid_from_secmod ()
{
PKCS11MOD_BASENAME=`basename $PKCS11MOD`
DISCOVERED_PKCS11MOD_LIBPATH=`modutil -rawlist -dbdir $1 2>/dev/null | grep "library=" | sed -e "s/.*\(library=\)\(\S*\).*/\2/" | grep $PKCS11MOD_BASENAME`
DISCOVERED_PKCS11MOD_NAME=`modutil -rawlist -dbdir $1 2>/dev/null | grep "library=$DISCOVERED_PKCS11MOD_LIBPATH" | sed -e "s/.*\(name=\)\([^\"]\S*\|\".*\"\).*/\2/" | sed -e "s/\"//g"`
modutil -force -delete "$DISCOVERED_PKCS11MOD_NAME" -dbdir $1
}

function disable_secmoddb_dir ()
{
remove_beleid_from_secmod $1
if [ $OO = "true" ] ; then
	if grep 'export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user' ~/.profile 2>/dev/null 1>&2; then
		grep -v 'export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user' ~/.profile > /tmp/.profile.eID
		mv /tmp/.profile.eID ~/.profile
	fi
	if grep 'export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user' ~/.bash_profile 2>/dev/null 1>&2; then
		grep -v 'export MOZILLA_CERTIFICATE_FOLDER=~/.ooo-2.0/user' ~/.bash_profile> /tmp/.bash_profile.eID
		mv /tmp/.bash_profile.eID ~/.bash_profile
	fi
fi
}

#######################################################

function check_secmoddb_dir ()
{
if [ -f ${1}/secmod.db ] ; then
	PKCS11MOD_BASENAME=`basename $PKCS11MOD`
	if modutil -rawlist -dbdir $1 2>/dev/null | grep "library=" | sed -e "s/.*\(library=\)\(\S*\).*/\2/" | grep $PKCS11MOD_BASENAME >/dev/null 2>&1; then
		return 0
	else
		return 1
	fi
else
	return 1
fi
}

#######################################################
 
function display_gui ()
{
local INDEX=0
while [ $INDEX -lt ${#application_array[*]} ] ; do
	if [ `echo "${application_array[$INDEX]}" | cut -d ' ' -f 1` = "Firefox" ] ; then
		echo -e "Firefox" ${application_array[$INDEX]##*.} ${application_status_array[$INDEX]}

	else
		echo -e ${application_array[$INDEX]} ${application_status_array[$INDEX]}
	fi
	INDEX=`expr $INDEX + 1`
done
}

#######################################################

function current_status ()
{

local INDEX=0
if [ -d "$FFBASE" ] ; then
	for PROFILE in `grep Path ${FFBASE}/profiles.ini | cut -d '=' -f 2`
	do
		application_array[$INDEX]="Firefox $PROFILE"
		application_home_array[$INDEX]="${FFBASE}/${PROFILE}"
		if check_secmoddb_dir "${FFBASE}/${PROFILE}" ; then
			application_status_array[$INDEX]="true"
		else
			application_status_array[$INDEX]="false"
		fi
		INDEX=`expr $INDEX + 1`
	done
else
	PROFILE="dummy.default"
        application_array[$INDEX]="Firefox $PROFILE"
        application_home_array[$INDEX]="${FFBASE}/${PROFILE}"
        application_status_array[$INDEX]="absent"
        INDEX=`expr $INDEX + 1`
fi

if [ -d "$OOBASE" ] ; then
	application_array[$INDEX]="OpenOffice"
	application_home_array[$INDEX]="$OOBASE"
	if check_secmoddb_dir "${OOBASE}" ; then
		application_status_array[$INDEX]="true"
	else
		application_status_array[$INDEX]="false"
	fi
	INDEX=`expr $INDEX + 1`
else
        application_array[$INDEX]="OpenOffice"
        application_home_array[$INDEX]="$OOBASE"
        application_status_array[$INDEX]="absent"
        INDEX=`expr $INDEX + 1`
fi

if [ -d "$EVOBASE" ] ; then
	application_array[$INDEX]="Evolution"
	application_home_array[$INDEX]="$EVOBASE"
	if check_secmoddb_dir "$EVOBASE" ; then
		application_status_array[$INDEX]="true"
	else
		application_status_array[$INDEX]="false"
	fi
	INDEX=`expr $INDEX + 1`
else
	application_array[$INDEX]="Evolution"
        application_home_array[$INDEX]="$EVOBASE"
        application_status_array[$INDEX]="absent"
	INDEX=`expr $INDEX + 1`

fi
}	

#######################################################

function changerevocheck () {
	if [ -f ~/.beidgui_ui.conf ] ; then
		if [ "$1" = "ocsp" ] ; then
			if sed -e "s/OCSP=.*/OCSP=$2/" < ~/.beidgui_ui.conf > /tmp/.beidgui_ui.conf.tmp ; then
				mv /tmp/.beidgui_ui.conf.tmp ~/.beidgui_ui.conf
			fi
		else
			if sed -e "s/CRL=.*/CRL=$2/" < ~/.beidgui_ui.conf > /tmp/.beidgui_ui.conf.tmp ; then
				mv /tmp/.beidgui_ui.conf.tmp ~/.beidgui_ui.conf
			fi
		fi
	fi
}

#######################################################

function usage () {
        exec 1>&2
        echo "Usage: $0 [ -c [0|1|2] | -o [0|1|2] | -r | -s | -p PIN -n (new)PIN | -e [ Evolution | OpenOffice | Firefox -f Profile ] | -d [ Evolution | OpenOffice | Firefox -f Profile ] ]"
        exit 255
}


#######################################################


if [ -f ~/.beidgui_ui.conf ] ; then
	:
else
	cat > ~/.beidgui_ui.conf <<EOF
SCReader=
Language=3
OCSP=0
CRL=0
Fonts=
Fontsize=10
EOF
fi

while getopts p:n:c:o:re:d:sf: OPT 
do
	case "$OPT" in
	c)
		if [ -z "$ARG" ] ; then
               		ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="crl"
		fi
                ;;
        o)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="ocsp"
		fi
                ;;
        r)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="showrevoke"
		fi
                ;;
        p)
                PIN="$OPTARG"
                ;;
        n)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="changepin"
		fi
                ;;
        e)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="enable"
		fi
                ;;
        d)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="disable"
		fi
                ;;
        s)
		if [ -z "$ARG" ] ; then
                       	ARG="$OPTARG"
		fi
		if [ -z "$CALL" ] ; then
                       	CALL="showapps"
		fi
                ;;
	f)
		PROFILE="$OPTARG"
		;;
        \?)
                usage
                ;;
	esac
done

case "$CALL" in
ocsp)
	case $ARG in
	0)
		changerevocheck ocsp 0
		;;
	1)
		changerevocheck ocsp 1
		;;
	2)
		changerevocheck ocsp 2
		;;
	*)
		usage
		;;
	esac
	;;
	
crl)
	case $ARG in
	0)
		changerevocheck crl 0
		;;
	1)
		changerevocheck crl 1
		;;
	2)
		changerevocheck crl 2
		;;
	*)
		usage
		;;
	esac
	;;

showrevoke)
	if [ -f ~/.beidgui_ui.conf ] ; then
		if OCSP_LINE=`grep -i 'ocsp=.' ~/.beidgui_ui.conf` ; then
			OCSP=`echo $OCSP_LINE | cut -d '=' -f 2`
			echo "ocsp=$OCSP"
		fi
		if CRL_LINE=`grep -i 'crl=.' ~/.beidgui_ui.conf` ; then
			CRL=`echo $CRL_LINE | cut -d '=' -f 2`
			echo "crl=$CRL"
		fi
	else
		echo "ocsp=0"
		echo "crl=0"
	fi
	;;

changepin)
	if [ -z "$PIN" ] ; then
                usage
        fi

	#if [ -z $2 -o -z $3 ] ; then
	#		exit 255
	#	fi
	if expect -f "$EXPECTFILE" "$PKCS11TOOL" "$PKCS11MOD" "$PIN" "$ARG" ; then
		exit 0
	else
		exit 255
	fi
	;;
showapps)
	current_status
	display_gui
	;;
enable)
	case $ARG in
	Firefox)
		if [ -z $PROFILE ] ; then
			usage
		fi
		
		if [ -d ${FFBASE}/*${PROFILE} ] ; then
			APPBASE=${FFBASE}/*${PROFILE}
			if check_secmoddb_dir $APPBASE ; then
				:
			else
				enable_secmoddb_dir $APPBASE 
			fi	
				
		else
			exit 255
		fi
		;;
	OpenOffice)
		OO=true
		if [ -d ${OOBASE} ] ; then
       	         	APPBASE=${OOBASE}
       	         	if check_secmoddb_dir $APPBASE ; then
				:
       	        	else
       	                	enable_secmoddb_dir $APPBASE 
       	         	fi
	
       	 	else
       	        	exit 255
       	 	fi
		;;
	Evolution)
		if [ -d ${EVOBASE} ] ; then
       	        	APPBASE=${EVOBASE}
       	        	if check_secmoddb_dir $APPBASE ; then
				:
       	         	else
       	                	enable_secmoddb_dir $APPBASE 
       	        	fi
	
       	 	else
       	        	exit 255
       	 	fi
		;;
	*)
		usage
		;;
	esac
	;;
disable)
	case $ARG in
	Firefox)
		if [ -z $PROFILE ] ; then
			usage
		fi
	
		if [ -d ${FFBASE}/*${PROFILE} ] ; then
			APPBASE=${FFBASE}/*${PROFILE}
			if check_secmoddb_dir $APPBASE ; then
				disable_secmoddb_dir $APPBASE
			else
				:
			fi	
			
		else
			exit 255
		fi
		;;
	OpenOffice)
		OO=true
		if [ -d ${OOBASE} ] ; then
                	APPBASE=${OOBASE}
                	if check_secmoddb_dir $APPBASE ; then
                       		disable_secmoddb_dir $APPBASE
                	else
				:
                	fi

        	else
                	exit 255
        	fi
		;;
	Evolution)
		if [ -d ${EVOBASE} ] ; then
                	APPBASE=${EVOBASE}
                	if check_secmoddb_dir $APPBASE ; then
                        	disable_secmoddb_dir $APPBASE
                	else
				:
                	fi

        	else
                	exit 255
        	fi
		;;
	*)
		usage	
		;;
	esac
	;;
*)
	exit 255
	;;
esac
